@@ -16,51 +16,74 @@ from django.conf import settings
 from django.views.debug import ExceptionReporter   
 import sys
 
+
 class APIException(Exception):
-    pass
+    def __init__(self):
+        self.headers = {}
+
+    def get_http_response_and_headers(self):
+        return http.HttpResponse(), self.headers
+
 
 class MethodNotAllowed(APIException):
     def __init__(self, allowed_methods=()):
-        self.status = 405
-        self.http_response = http.HttpResponseNotAllowed(allowed_methods)
+        self.status=405
+        self.allowed_methods = allowed_methods
+        self.headers = {}
+
+    def get_http_response_and_headers(self):
+        return http.HttpResponseNotAllowed(self.allowed_methods), self.headers
+
 
 class BadRequest(APIException):
     def __init__(self, errors=None):
         self.status = 400
         self.errors = errors
+        self.body, self.headers = errors and serialize(errors) or ('', {})
+
+    def get_http_response_and_headers(self):
+        return http.HttpResponseBadRequest(self.body), self.headers
 
-        errors, headers = errors and serialize(errors) or ('', {})
-        self.http_response = http.HttpResponseBadRequest(errors)
-        for key, value in headers.items():
-            self.http_response[key] = value
 
 class Gone(APIException):
     def __init__(self):
         self.status = 410
-        self.http_response = http.HttpResponseGone()
+        self.headers = {}
+
+    def get_http_response_and_headers(self):        
+        return http.HttpResponseGone(), self.headers
+
 
 class Unprocessable(APIException):
     def __init__(self, errors=None):
         self.status = 422
         self.errors = errors
+        self.body, self.headers = errors and serialize(errors) or ('', {})
+
+    def get_http_response_and_headers(self):
+        return http.HttpResponse(self.body, status=self.status), self.headers
 
-        errors, headers = errors and serialize(errors) or ('', {})
-        self.http_response = http.HttpResponse(errors, status=422)
-        for key, value in headers.items():
-            self.http_response[key] = value
 
 class UnsupportedMediaType(APIException):
     def __init__(self):
         self.status = 415
-        self.http_response = http.HttpResponse(status=415)
+        self.headers = {}
+
+    def get_http_response_and_headers(self):        
+        return http.HttpResponse(status=self.status), self.headers
+
 
 class NotImplemented(APIException):
     def __init__(self):
         self.status = 501
-        self.http_response = http.HttpResponse(status=501)
+        self.headers = {}
+    
+    def get_http_response_and_headers(self):
+        return http.HttpResponse(status=self.status), self.headers
+
 
 class OtherException(Exception):
-    def __init__(self, e, request):
+    def __init__(self, request):
         """
         @param e: Some Exception instance
 
@@ -70,9 +93,13 @@ class OtherException(Exception):
         administrator.
         """
         self.status = 500
+        self.request = request
+        self.headers = {}
+
+    def get_http_response_and_headers(self):
         exc_type, exc_value, traceback = sys.exc_info()
         reporter = ExceptionReporter(
-            request, 
+            self.request, 
             exc_type, 
             exc_value,
             traceback.tb_next
@@ -80,22 +107,29 @@ class OtherException(Exception):
         html = reporter.get_traceback_html()
 
         if settings.DEBUG:
-            self.http_response = http.HttpResponseServerError(
+            http_response = http.HttpResponseServerError(
                 html,
                 content_type='text/html; charset=utf-8'
             )
+            self.headers['content-type'] = 'text/html; charset=utf-8'
         else:
-            self.http_response = http.HttpResponseServerError()
-            # and send email
+            http_response = http.HttpResponseServerError()
+            # TODO: and send email
 
+        return http_response, self.headers
+
+
+# TODO: This function is obsolete, since the
+# HandlerControlFlow.handle_exception is used instead. Delete it,and remove its
+# corresponding tests.
 def handle_exception(e, request):
     # If exception ``e`` is of type APIException, we simply return the
     # corresponding HttpResponse object.
     if isinstance(e, APIException):
-        return e.http_response
+        return e.get_http_response_and_headers()
 
     # Else, we make it an OtherException, and return a HttpResponse 500 object
     else:
-        exc = OtherException(e, request)
-        return exc.http_response
+        exc = OtherException(request)
+        return exc.get_http_response_and_headers()
 
